/**
 * Kivu4j. Copyright (C) 2010 . see<http://kivu4j.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */

package net.sourceforge.kivu4j.utils.struts;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.util.zip.GZIPOutputStream;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sourceforge.kivu4j.utils.json.google.JsonUtilImpl;

import org.apache.commons.lang.StringUtils;
import org.apache.struts2.StrutsConstants;
import org.apache.struts2.StrutsStatics;
import org.apache.struts2.dispatcher.StrutsResultSupport;

import com.opensymphony.xwork2.ActionContext;
import com.opensymphony.xwork2.ActionInvocation;
import com.opensymphony.xwork2.inject.Inject;
import com.opensymphony.xwork2.util.ValueStack;


/**
 * json结果集
 *
 * 
 * @author Lucky Yang
 * @see org.apache.struts2.dispatcher.StrutsResultSupport
 */
public class JSONResult extends StrutsResultSupport {
	public static final String DEFAULT_ROOT = "responseData";
	/**
	 * 
	 */
	private static final long serialVersionUID = -6279758293265627349L;
	 
	private String root;
	private String callbackParameter;
	private boolean enableGZIP = false;
	private String defaultEncoding = "UTF-8";
	private boolean noCache = false;
	
    public void setCallbackParameter(String callbackParameter) {
        this.callbackParameter = callbackParameter;
    }

    public String getCallbackParameter() {
        return callbackParameter;
    }

	public String getRoot() {
		return StringUtils.defaultString(root,DEFAULT_ROOT);
	}

	public void setRoot(String root) {
		this.root = root;
	}

	public boolean isNoCache() {
        return noCache;
    }
    
    public void setNoCache(boolean noCache) {
        this.noCache = noCache;
    }
    
    @Inject(StrutsConstants.STRUTS_I18N_ENCODING)
    public void setDefaultEncoding(String val) {
        this.defaultEncoding = val;
    }
    
    protected String getEncoding() {
        String encoding = this.defaultEncoding;

        if (encoding == null) {
            encoding = System.getProperty("file.encoding");
        }

        if (encoding == null) {
            encoding = "UTF-8";
        }

        return encoding;
    }

	@Override
	protected void doExecute(String finalLocation, ActionInvocation invocation) throws Exception {
        ActionContext actionContext = invocation.getInvocationContext();
        HttpServletRequest request 
        	= (HttpServletRequest)actionContext.get(StrutsStatics.HTTP_REQUEST);
        HttpServletResponse response 
        	= (HttpServletResponse) actionContext.get(StrutsStatics.HTTP_RESPONSE);

        try {
            Object rootObject;
            rootObject = findRootObject(invocation);
            
            writeToResponse(
            		response
            		, createJSONString(request, rootObject)
            		, enableGzip(request)
            );
        } catch (IOException exception) {
            throw exception;
        }		
	}

    private Object findRootObject(ActionInvocation invocation){
        Object rootObject;
        if (this.getRoot() != null) {
            ValueStack stack = invocation.getStack();
            rootObject = stack.findValue(this.getRoot());
        } else {
            rootObject = invocation.getStack().peek(); // model overrides action
        }
        return rootObject;
    }
    
    private String createJSONString(HttpServletRequest request, Object rootObject){
        String json = JsonUtilImpl.getInstance().toJson(rootObject);
        json = addCallbackIfApplicable(request, json);
        return json;
    }
    
    private boolean enableGzip(HttpServletRequest request) {
        return enableGZIP && this.isGzipInRequest(request);
    }
    
    private boolean isGzipInRequest(HttpServletRequest request) {
        String header = request.getHeader("Accept-Encoding");
        return (header != null) && (header.indexOf("gzip") >= 0);
    }

    private void writeJSONToResponse(String json, HttpServletResponse response
    		, boolean isNoCache, boolean isGzip) throws IOException {
        response.setContentType("application/json;charset=" + this.getEncoding());

        if (isNoCache) {
            response.setHeader("Cache-Control", "no-cache");
            response.setHeader("Expires", "0");
            response.setHeader("Pragma", "No-cache");
        }

        if (isGzip) {
            response.addHeader("Content-Encoding", "gzip");
            GZIPOutputStream out = null;
            InputStream in = null;
            try {
                out = new GZIPOutputStream(response.getOutputStream());
                in = new ByteArrayInputStream(json.getBytes(this.getEncoding()));
                byte[] buf = new byte[1024];
                int len;
                while ((len = in.read(buf)) > 0) {
                    out.write(buf, 0, len);
                }
            } finally {
                if (in != null)
                    in.close();
                if (out != null) {
                    out.finish();
                    out.close();
                }
            }

        } else {
            response.setContentLength(json.getBytes(this.getEncoding()).length);
            PrintWriter out = response.getWriter();
            out.print(json);
        }
    }
    
    protected void writeToResponse(HttpServletResponse response, String json, boolean gzip) throws IOException {
        this.writeJSONToResponse(json, response, this.isNoCache(), gzip);
    }  
    
    protected String addCallbackIfApplicable(HttpServletRequest request, String json) {
        if ((callbackParameter != null) && (callbackParameter.length() > 0)) {
            String callbackName = request.getParameter(callbackParameter);
            if ((callbackName != null) && (callbackName.length() > 0))
                json = callbackName + "(" + json + ")";
        }
        return json;
    }    
}
